@@ -529,7 +529,7 @@ class StandardParser:
 # INSTANCES
 
 from logics.instances.propositional.languages import classical_infinite_language_with_sent_constants, \
-    modal_infinite_language_with_sent_constants
+    LFI_language, modal_infinite_language_with_sent_constants
 
 
 classical_parse_replacement_dict = {
@@ -539,6 +539,7 @@ classical_parse_replacement_dict = {
 
     # For conjunction
     '&': '∧',
+    '^': '∧',
     ' and ': '∧',  # notice the spaces before and after 'and'
 
     # For disjunction
@@ -582,6 +583,19 @@ classical_parser = StandardParser(language=classical_infinite_language_with_sent
                                   infix_cts=['∧', '∨', '→', '↔'],
                                   comma_separator=',', inference_separator='/', derivation_step_separator=';')
 
+LFI_replacement_dict = copy(classical_parse_replacement_dict)
+LFI_replacement_dict.update({
+    "inc ": "•",
+    "inconsistent ": "•",
+    "con ": "◦",
+    "consistent ": "◦",
+    "°": "◦",
+})
+LFI_parser = StandardParser(language=LFI_language,
+                            parse_replacement_dict=LFI_replacement_dict,
+                            infix_cts=['∧', '∨', '→', '↔'],
+                            comma_separator=',', inference_separator='/', derivation_step_separator=';')
+
 modal_replacement_dict = copy(classical_parse_replacement_dict)
 modal_replacement_dict.update({
     'box ': '□',
